@@ -1,5 +1,33 @@
 DEBUG=False
 
+try:
+    from PyQt5 import QtCore
+    from PyQt5.QtCore import QObject
+
+    from PyQt5.QtWidgets import (
+            QApplication,
+            QWidget,
+            QMainWindow,
+            QMessageBox,
+            QTabWidget,
+            QVBoxLayout,
+            QHBoxLayout,
+            QLabel,
+            QLineEdit,
+            QPushButton,
+            QScrollArea,)
+except ImportError as e:
+    print("------------------------------")
+    print("|           ERROR            |")
+    print("------------------------------")
+    print("| Error importing PyQt5.     |")
+    print("| Install PyQt5:             |")
+    print("|    'pip install PyQt5'     |")
+    print("|            or              |")
+    print("|    'conda install pyqt'    |")
+    print("------------------------------")
+    raise e
+
 import pandas
 
 import matplotlib
@@ -10,24 +38,8 @@ from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavBar
 
 import re
 
-from PyQt5 import QtCore
-from PyQt5.QtCore import QObject
-
-from PyQt5.QtWidgets import (
-        QApplication,
-        QWidget,
-        QMainWindow,
-        QMessageBox,
-        QTabWidget,
-        QVBoxLayout,
-        QHBoxLayout,
-        QLabel,
-        QLineEdit,
-        QPushButton,
-        QScrollArea,)
-
 class TagInfoRule():
-    def __init__(self,expr,color,sub=r'\1'):
+    def __init__(self,expr,color=None,sub=r'\1'):
         self.expr = expr
         self.rexpr = re.compile(expr)
         self.sub = sub
@@ -68,22 +80,27 @@ class TagInfo():
     '''
 
     taginfo_rules = [
-        TagInfoRule(expr=r'(.*)\.PV',color='C0'),
-        TagInfoRule(expr=r'(.*)\.MEAS',color='C0'),
-        TagInfoRule(expr=r'(.*)\.SP',color='C1'),
-        TagInfoRule(expr=r'(.*)\.SPT',color='C1'),
-        TagInfoRule(expr=r'(.*)\.READVALUE',color='C0'),
-        TagInfoRule(expr=r'(.*)\.SSVALUE',color='cyan'),
-        TagInfoRule(expr=r'(.*)\.HIGHLIMIT',color='red'),
-        TagInfoRule(expr=r'(.*)\.LOWLIMIT',color='red'),
+        TagInfoRule(expr=r'(.*)\.PV$',color='C0'),
+        TagInfoRule(expr=r'(.*)\.MEAS$',color='C0'),
+        TagInfoRule(expr=r'(.*)\.SP$',color='C1'),
+        TagInfoRule(expr=r'(.*)\.SPT$',color='C1'),
     ]
 
     def __init__(self,tagname):
+        '''
+        Constructor
+
+        Parameters:
+        -----------
+        tagname : str
+            name of tag
+        '''
+
         self.name = tagname
         self.plotinfo = None # points to a plotinfo if tag is plotted
 
         self.groupid = None
-        self.color = 'black'
+        self.color = None
         for rule in self.taginfo_rules:
             match, gid = rule.get_groupid(self.name)
             if match:
@@ -103,42 +120,127 @@ class PlotManager(QObject):
 
     '''
 
-    def __init__(self,df,plot_window,parent=None):
+    def __init__(self,parent=None):
         QObject.__init__(self,parent)
-        '''
-        Parameters:
-        -----------
-        df : pandas.core.frame.DataFrame
-            dataframe with plot data
-        plot_window : PlotWindow
-            window where plotting happens
 
-        '''
-        self._df = df
-        self._fig = plot_window.fig
-        self._canvas = plot_window.canvas
+        self._df = None
+        self.plot_window = PlotWindow()
+        self.plot_window.home_zoom_signal.connect(self.home_zoom)
 
         self._plotinfo = [] # list of info about plot
         self._groupid_plots = {} # dictionary of plotted groupids
         self._taginfo = {} # dictionary of tags
 
+    def set_dataframe(self,df):
+        self.clear_all_plots()
+
+        self._df = df
+        self._taginfo.clear()
+
         for tag in self._df:
-            self._taginfo[tag] = TagInfo(tag)
+            # Check if we can plot the tag
+            dt = self._df[tag].dtype
+            if dt in (float,int,bool,'int64'):
+                self._taginfo[tag] = TagInfo(tag)
+            else:
+                if DEBUG:
+                    print('Tag {} is not plottable'.format(tag))
+                    print('    dtype is {}'.format(dt))
+                continue
 
 
-    def replot(self,plotinfo):
+    def get_tagtools(self):
+        '''
+        Get tagtools of all validated _taginfos
+        '''
+        tools = [ TagTool(t) for t in self._taginfo ]
+        for tool in tools:
+            tool.add_remove_plot.connect(self.add_remove_plot)
+
+        return tools
+            
+
+    @QtCore.pyqtSlot()
+    def home_zoom(self):
+        '''
+        Sets the zoom level to default.
+        '''
+        if DEBUG:
+            print('PlotManager::home_clicked()')
+
+        try:
+            #plt.margins(0,0.05)
+            if len(self._plotinfo) > 0:
+                for pi in self._plotinfo:
+                    pi.ax.autoscale(axis='x',tight=True)
+                    pi.ax.autoscale(axis='y',tight=False)
+            self.plot_window.canvas.draw()
+
+            
+            
+        except Exception as e:
+            sys.stderr.write(e)
+
+    @QtCore.pyqtSlot()
+    def clear_all_plots(self):
+        if DEBUG:
+            print('PlotManager::clear_all_plots()')
+
+        try:
+            
+            while len(self._plotinfo) > 0:
+                p = self._plotinfo.pop()
+                for t in p.tagnames:
+                    self._taginfo[t].plotinfo = None
+            
+            self._plotinfo.clear()
+            self._groupid_plots.clear()
+            self.plot_window.fig.clear()
+            self.plot_window.toolbar._nav_stack.clear()
+            self.plot_window.canvas.draw()
+        except Exception as e:
+            sys.stderr.write(e)
+            
+
+  
+    @QtCore.pyqtSlot()
+    def refresh(self):
+        try:
+            if DEBUG:
+                print("PlotManager::refresh()")
+
+            for pi in self._plotinfo:
+                self.replot(pi)
+
+            self.plot_window.toolbar._nav_stack.clear()
+            self.plot_window.fig.tight_layout()
+            self.plot_window.canvas.draw()
+        except Exception as e:
+            sys.stderr.write(e)
+
+    def replot(self,plotinfo,save_xlim=False):
         '''
         Replot the ax in plotinfo.  Used when adding/removing tags
         '''
-        plotinfo.ax.clear()
-        color = []
-        for tag in plotinfo.tagnames:
-            color.append( self._taginfo[tag].color )
-        self._df[plotinfo.tagnames].plot(
-            color=color,
-            ax=plotinfo.ax)
 
-        self._fig.tight_layout()
+        if save_xlim:
+            xlim = plotinfo.ax.get_xlim()
+
+        plotinfo.ax.clear()
+
+        color = [ self._taginfo[t].color for t in plotinfo.tagnames ]
+        self._df.plot(
+            y=plotinfo.tagnames,
+            color=color,
+            ax=plotinfo.ax,
+            legend=True,
+            include_bool=True,
+        )
+
+        if save_xlim:
+            plotinfo.ax.set_xlim(xlim)
+
+
 
     def add_plot(self,tag):
         taginfo = self._taginfo[tag]
@@ -155,6 +257,20 @@ class PlotManager(QObject):
             plotinfo = self._groupid_plots[groupid]
             plotinfo.tagnames.append(tag)
             taginfo.plotinfo = plotinfo
+
+            # Only plot new tag so that zoom doesn't change
+            if DEBUG:
+                print("Adding tag to axis")
+
+            xlim = plotinfo.ax.get_xlim()
+
+            self._df[tag].plot(color=taginfo.color,
+                               ax=plotinfo.ax,
+                               legend=True,
+                               include_bool=True)
+
+            plotinfo.ax.set_xlim(xlim)
+
         else:
             nplots = len(self._plotinfo)
 
@@ -165,14 +281,18 @@ class PlotManager(QObject):
                 sharex = None
 
             # resize existing axes
+            if DEBUG:
+                print("Resize existing axes")
             gs = matplotlib.gridspec.GridSpec(nplots+1,1)
             for i in range(nplots):
-                self._plotinfo[i].ax.set_position( gs[i].get_position(self._fig) )
+                self._plotinfo[i].ax.set_position( gs[i].get_position(self.plot_window.fig) )
                 self._plotinfo[i].ax.set_subplotspec( gs[i] )
 
-
-            ax = self._fig.add_subplot(
+            if DEBUG:
+                print("Create new axes")
+            ax = self.plot_window.fig.add_subplot(
                 nplots+1,1,nplots+1,
+                label=groupid,
                 sharex=sharex
             )
             plotinfo = PlotInfo(tag,groupid,ax)
@@ -180,8 +300,15 @@ class PlotManager(QObject):
             self._taginfo[tag].plotinfo = plotinfo
             if groupid:
                 self._groupid_plots[groupid] = plotinfo
-        
-        self.replot(plotinfo)
+       
+            if DEBUG:
+                print("Replotting new axis")
+
+            self.replot(plotinfo,save_xlim=(sharex!=None))
+
+            if DEBUG:
+                print("Clearing navstack")
+            self.plot_window.toolbar._nav_stack.clear()
 
 
     def remove_plot(self,tag):
@@ -200,7 +327,7 @@ class PlotManager(QObject):
                 print("Remaining tags:")
                 print(plotinfo.tagnames)
 
-            self.replot(plotinfo)
+            self.replot(plotinfo,save_xlim=True)
 
         else:
             # remove whole axes
@@ -214,9 +341,14 @@ class PlotManager(QObject):
             nplots = len(self._plotinfo)
             gs = matplotlib.gridspec.GridSpec(nplots,1)
             for i in range(nplots):
-                self._plotinfo[i].ax.set_position( gs[i].get_position(self._fig) )
+                self._plotinfo[i].ax.set_position( gs[i].get_position(self.plot_window.fig) )
                 self._plotinfo[i].ax.set_subplotspec( gs[i] )
 
+            if len(self._plotinfo) == 0:
+                if DEBUG:
+                    print("no more plots left")
+                    self.plot_window.toolbar._nav_stack.clear()
+
         taginfo.plotinfo = None
 
 
@@ -236,19 +368,16 @@ class PlotManager(QObject):
         if DEBUG:
             print("PlotManager::add_remove_plot({},{})".format(tag,add))
 
-        interactive = plt.isinteractive()
-        if interactive:
-            plt.ioff()
+        try:
+            if add:
+                self.add_plot(tag)
+            else:
+                self.remove_plot(tag)
 
-        if add:
-            self.add_plot(tag)
-        else:
-            self.remove_plot(tag)
+            self.plot_window.canvas.draw()
+        except Exception as e:
+            sys.stderr.write(e)
 
-        self._canvas.draw()
-
-        if interactive:
-            plt.ion()
 
     @QtCore.pyqtSlot()
     def showme(self):
@@ -306,8 +435,13 @@ class PlotWindow(QWidget):
     '''
     A single plot window.
 
-
     '''
+
+    # signal is emitted when home is clicked but navstack
+    # is empty
+    home_zoom_signal = QtCore.Signal()
+    
+
     def __init__(self,parent=None):
         QWidget.__init__(self,parent)
 
@@ -320,6 +454,24 @@ class PlotWindow(QWidget):
         layout.addWidget(self.canvas)
         self.setLayout(layout)
 
+        # find toolbar's home button
+        home_action = None
+        for action in self.toolbar.actions():
+            if action.text() == 'Home':
+                home_action = action
+                break
+
+        if home_action == None:
+            sys.stderr.write('Home action in Qt Navbar not found')
+        else:
+            home_action.triggered.connect(self.home_clicked)
+
+    @QtCore.pyqtSlot()
+    def home_clicked(self):
+        if self.toolbar._nav_stack.empty():
+            self.home_zoom_signal.emit()
+                
+
 
 
 class ToolPanel(QWidget):
@@ -334,6 +486,8 @@ class ToolPanel(QWidget):
     '''
 
     showme_clicked = QtCore.Signal()
+    clear_click_signal = QtCore.Signal()
+    refresh_click_signal = QtCore.Signal()
 
     def __init__(self,parent=None):
         QWidget.__init__(self,parent)
@@ -342,6 +496,11 @@ class ToolPanel(QWidget):
         showme_button = QPushButton('Show Me')
         showme_button.clicked.connect(self.showme_clicked)
 
+        clear_button = QPushButton("Clear")
+        clear_button.clicked.connect(self.clear_clicked)
+
+        refresh_button = QPushButton("Refresh")
+        refresh_button.clicked.connect(self.refresh_click_signal)
 
         self.filter_textbox = QLineEdit()
         self.filter_textbox.textChanged.connect(self.filter_changed)
@@ -372,28 +531,53 @@ class ToolPanel(QWidget):
         scroll_widget.setLayout(scroll_layout)
 
         main_layout = QVBoxLayout()
+        main_layout.setContentsMargins(0,0,0,0)
+        main_layout.setSpacing(0)
         main_layout.addWidget(showme_button)
+        main_layout.addWidget(clear_button)
         main_layout.addWidget(self.filter_textbox)
         main_layout.addWidget(scroll_area)
+        main_layout.addWidget(refresh_button)
         self.setLayout(main_layout)
 
-    def add_tagtool(self,tagtool):
-        if tagtool.parent() != self:
-            if DEBUG:
-                print("Changing parent of TagTool")
-                print("{} -> {}".format(tagtool.parent,self))
-            tagtool.parent = self
+    def add_tagtools(self,tagtools):
+        for t in tagtools:
+            self._tools.append(t)
+            self.tool_layout.addWidget(t)
 
-        self._tools.append(tagtool)
-        self.tool_layout.addWidget(tagtool)
+            assert t.parent() != None, 'tagtool has no parent'
 
-    QtCore.pyqtSlot(str)
+    def remove_tagtools(self):
+        while True:
+            try:
+                tool = self._tools.pop()
+                tool.setParent(None)
+            except IndexError:
+                break
+
+
+    @QtCore.pyqtSlot(str)
     def filter_changed(self,filter_text):
         for tool in self._tools:
-            if filter_text in tool.name:
+            if filter_text.lower() in tool.name.lower():
                 tool.show()
             else:
                 tool.hide()
+
+    @QtCore.pyqtSlot()
+    def clear_clicked(self):
+        if DEBUG:
+            print("ToolPanel::clear_clicked")
+
+        for tool in self._tools:
+            try:
+                tool.reset()
+            except Exception as e:
+                sys.stderr.write(e)
+
+        if DEBUG:
+            print("Emit clear clicked")
+        self.clear_click_signal.emit()
         
 
 
@@ -409,7 +593,7 @@ class TagTool(QWidget):
 
     add_remove_plot = QtCore.Signal(str,bool)
 
-    def __init__(self,name,parent_toollist):
+    def __init__(self,name):
         '''
         Constructing a tool also adds it to its parent ToolPanel's layout
 
@@ -421,8 +605,7 @@ class TagTool(QWidget):
             Qt parent, this tool is also added to the toollist's layout
 
         '''
-        QWidget.__init__(self,parent_toollist)
-
+        QWidget.__init__(self)
 
         self.name = name
         self.plot_button = QPushButton(name)
@@ -434,16 +617,19 @@ class TagTool(QWidget):
         layout.setSpacing(0)
         layout.addWidget(self.plot_button,1)
 
-
         self.setLayout(layout)
 
-        parent_toollist.add_tagtool(self)
 
     @QtCore.pyqtSlot(bool)
     def plot_clicked(self,is_clicked):
         self.add_remove_plot.emit(self.name,is_clicked)
 
-def add_grouping_rule(expr,color,sub=r'\1'):
+    def reset(self):
+        self.blockSignals(True)
+        self.plot_button.setChecked(False)
+        self.blockSignals(False)
+
+def add_grouping_rule(expr,color=None,sub=r'\1'):
     '''
     Add a rule to group trends.
 
@@ -530,6 +716,28 @@ def add_grouping_rule(expr,color,sub=r'\1'):
         TagInfoRule(expr,color,sub)
     )
 
+def remove_grouping_rules(index=None):
+    '''
+    Remove grouping rules.
+    Note: Rules are only applied when the dataframe is set, you need to set the
+    dataframe again for this change to apply.
+
+    Parameters:
+    -----------
+    index : int, optional
+        Index of rule to remove. If None, clear all the grouping rules.
+
+    '''
+    global _isInit
+
+    if _isInit:
+        print("Warning: changing the grouping rules will not have an effect",
+              "until you call set_dataframe() again.")
+    if index == None:
+        TagInfo.taginfo_rules.clear()
+    else:
+        TagInfo.taginfo_rules.pop(index)
+
 def print_grouping_rules():
     '''
     Print all grouping rules.
@@ -542,8 +750,12 @@ def print_grouping_rules():
             sub = 'None'
         else:
             sub = rule.sub
+        if rule.color == None:
+            col = 'None'
+        else:
+            col = rule.color
         print("{:<3} {:<60} {:^10} {}"\
-            .format(i, rule.expr, rule.color, sub )
+            .format(i, rule.expr, col, sub )
         )
         
 def set_dataframe(df):
@@ -560,39 +772,18 @@ def set_dataframe(df):
     global _isInit
     global _df
     global main_window
+    global tool_panel
+    global plot_window
+    global plot_manager
 
     if _isInit:
-        print("Dataframe is already initialised")
-        print("TODO: enable re-initialising")
-        return
+        tool_panel.remove_tagtools()
 
-    interactive = plt.isinteractive()
-    if interactive:
-        plt.ioff()
+    plot_manager.set_dataframe(df)
+    tool_panel.add_tagtools( plot_manager.get_tagtools() )
 
-    main_window = QWidget()
-    plot_window = PlotWindow(main_window)
-    tool_list = ToolPanel(main_window)
-
-    layout = QHBoxLayout()
-    layout.addWidget(tool_list,0)
-    layout.addWidget(plot_window,1)
-    main_window.setLayout(layout)
-
-    plot_manager = PlotManager(df,plot_window,main_window)
-
-    showme_dialog = QMessageBox(main_window)
-
-
-    tool_list.showme_clicked.connect(plot_manager.showme)
-
-    for tag in df.columns:
-        tool = TagTool(tag,tool_list)
-        tool.add_remove_plot.connect(plot_manager.add_remove_plot)
     _isInit = True
 
-    if interactive:
-        plt.ion()
 
 def show():
     '''
@@ -616,13 +807,36 @@ def show():
         app.exec_()
         app.exit()
 
+def set_exec_on_show(on=True):
+    '''
+    Set whether Qt app .exec function should be called on show().  When
+    proc_plot is used in a jupyter notebook with %matplotlib qt magic then the
+    gui loop is already running and starting it again will break the app.
+    proc_plot tries to figure it out automatically but you can override the
+    setting with this function.
+
+    Parameters:
+    -----------
+    on : bool, optional
+        set to False to disable runnnig app.exec.
+
+    '''
+    global _execApp
+    _execApp = on
 
 _isInit = False # has the window been initialised with a dataframe?
-_execApp = False # if started with qt, gui loop is running
-if plt.get_backend().lower() == 'nbagg':
-    _execApp = True
-    if DEBUG:
-        print("Explicitly running gui loop for nbAgg")
+_execApp = True # if started with qt, gui loop is running
+
+if DEBUG:
+    print("Backend: ", plt.get_backend())
+
+if (plt.get_backend().lower() == 'qt5agg' and
+    plt.isinteractive() ):
+    # looks like you are running a jupyter notebook with %matplotlib qt
+    _execApp = False
+    print("It looks like you are running a jupyter notebook with " \
+         +"%matplotlib qt magic.\nThe gui loop is disabled, if you " \
+         +"want to enable it, use proc_plot.set_exec_on_show()")
 
 app = QtCore.QCoreApplication.instance()
 if app is None:
@@ -630,6 +844,24 @@ if app is None:
     if DEBUG:
         print("app was None")
 
-main_window = None
+interactive = plt.isinteractive()
+if interactive:
+    plt.ioff()
+
+main_window = QWidget()
+plot_manager = PlotManager(main_window)
+tool_panel = ToolPanel(main_window)
+
+tool_panel.showme_clicked.connect(plot_manager.showme)
+tool_panel.clear_click_signal.connect(plot_manager.clear_all_plots)
+tool_panel.refresh_click_signal.connect(plot_manager.refresh)
+
+layout = QHBoxLayout()
+layout.addWidget(tool_panel,0)
+layout.addWidget(plot_manager.plot_window,1)
+main_window.setLayout(layout)
+#del layout
 
 
+if interactive:
+    plt.ion()
